/**
 * ExcelUtil.java 2016年7月25日
 */
package cn.com.countrygarden.bi.main.util;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.google.common.collect.Lists;


public class ExcelUtils {
	/**
	 * 
	 * @param title 显示的导出表的标题
	 * @param rowName  导出表的列名
	 * @param dataList  数据集
	 * @return
	 */
	public static HSSFWorkbook export(String title, String[] rowName, List<Object[]> dataList) {
		HSSFWorkbook workbook = new HSSFWorkbook(); // 创建工作簿对象
		try {
			HSSFSheet sheet = workbook.createSheet(title); // 创建工作表

			// 产生表格标题行
			HSSFRow rowm = sheet.createRow(0);
			HSSFCell cellTiltle = rowm.createCell(0);

			// sheet样式定义【getColumnTopStyle()/getStyle()均为自定义方法 - 在下面 - 可扩展】
			HSSFCellStyle columnTopStyle = getColumnTopStyle(workbook);// 获取列头样式对象
			HSSFCellStyle style = getStyle(workbook); // 单元格样式对象

			sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, (rowName.length - 1)));
			cellTiltle.setCellStyle(columnTopStyle);
			cellTiltle.setCellValue(title);

			// 定义所需列数
			int columnNum = rowName.length;
			HSSFRow rowRowName = sheet.createRow(2); // 在索引2的位置创建行(最顶端的行开始的第二行)

			// 将列头设置到sheet的单元格中
			for (int n = 0; n < columnNum; n++) {
				HSSFCell cellRowName = rowRowName.createCell(n); // 创建列头对应个数的单元格
				cellRowName.setCellType(HSSFCell.CELL_TYPE_STRING); // 设置列头单元格的数据类型
				HSSFRichTextString text = new HSSFRichTextString(rowName[n]);
				cellRowName.setCellValue(text); // 设置列头单元格的值
				cellRowName.setCellStyle(columnTopStyle); // 设置列头单元格样式
			}

			// 将查询出的数据设置到sheet对应的单元格中
			for (int i = 0; i < dataList.size(); i++) {

				Object[] obj = dataList.get(i);// 遍历每个对象
				HSSFRow row = sheet.createRow(i + 3);// 创建所需的行数

				for (int j = 0; j < obj.length; j++) {
					HSSFCell cell = null; // 设置单元格的数据类型
					//						if (j == 0) {
					//							cell = row.createCell(j, HSSFCell.CELL_TYPE_NUMERIC);
					//							cell.setCellValue(i + 1);
					//						} else {
					cell = row.createCell(j, HSSFCell.CELL_TYPE_STRING);
					if (obj[j] != null) {
						cell.setCellValue(obj[j].toString()); // 设置单元格的值
					} else {
						cell.setCellValue("");
					}
					//						}
					cell.setCellStyle(style); // 设置单元格样式
				}
			}
			// 让列宽随着导出的列长自动适应
			for (int colNum = 0; colNum < columnNum; colNum++) {
				int columnWidth = sheet.getColumnWidth(colNum) / 256;
				for (int rowNum = 0; rowNum < sheet.getLastRowNum(); rowNum++) {
					HSSFRow currentRow;
					// 当前行未被使用过
					if (sheet.getRow(rowNum) == null) {
						currentRow = sheet.createRow(rowNum);
					} else {
						currentRow = sheet.getRow(rowNum);
					}
					if (currentRow.getCell(colNum) != null) {
						HSSFCell currentCell = currentRow.getCell(colNum);
						if (currentCell.getCellType() == HSSFCell.CELL_TYPE_STRING) {
							int length = currentCell.getStringCellValue().getBytes().length;
							if (columnWidth < length) {
								columnWidth = length;
							}
						}
					}
				}
				if (colNum == 0) {
					sheet.setColumnWidth(colNum, (columnWidth - 2) * 256);
				} else {
					sheet.setColumnWidth(colNum, (columnWidth + 4) * 256);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return workbook;

	}

	/*
	 * 列头单元格样式
	 */
	private static HSSFCellStyle getColumnTopStyle(HSSFWorkbook workbook) {

		// 设置字体
		HSSFFont font = workbook.createFont();
		// 设置字体大小
		font.setFontHeightInPoints((short) 11);
		// 字体加粗
		font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
		// 设置字体名字
		font.setFontName("Courier New");
		// 设置样式;
		HSSFCellStyle style = workbook.createCellStyle();
		// 设置底边框;
		style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
		// 设置底边框颜色;
		style.setBottomBorderColor(HSSFColor.BLACK.index);
		// 设置左边框;
		style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
		// 设置左边框颜色;
		style.setLeftBorderColor(HSSFColor.BLACK.index);
		// 设置右边框;
		style.setBorderRight(HSSFCellStyle.BORDER_THIN);
		// 设置右边框颜色;
		style.setRightBorderColor(HSSFColor.BLACK.index);
		// 设置顶边框;
		style.setBorderTop(HSSFCellStyle.BORDER_THIN);
		// 设置顶边框颜色;
		style.setTopBorderColor(HSSFColor.BLACK.index);
		// 在样式用应用设置的字体;
		style.setFont(font);
		// 设置自动换行;
		style.setWrapText(false);
		// 设置水平对齐的样式为居中对齐;
		style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
		// 设置垂直对齐的样式为居中对齐;
		style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);

		return style;

	}

	/*
	 * 列数据信息单元格样式
	 */
	private static HSSFCellStyle getStyle(HSSFWorkbook workbook) {
		// 设置字体
		HSSFFont font = workbook.createFont();
		// 设置字体大小
		// font.setFontHeightInPoints((short)10);
		// 字体加粗
		// font.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
		// 设置字体名字
		font.setFontName("Courier New");
		// 设置样式;
		HSSFCellStyle style = workbook.createCellStyle();
		// 设置底边框;
		style.setBorderBottom(HSSFCellStyle.BORDER_THIN);
		// 设置底边框颜色;
		style.setBottomBorderColor(HSSFColor.BLACK.index);
		// 设置左边框;
		style.setBorderLeft(HSSFCellStyle.BORDER_THIN);
		// 设置左边框颜色;
		style.setLeftBorderColor(HSSFColor.BLACK.index);
		// 设置右边框;
		style.setBorderRight(HSSFCellStyle.BORDER_THIN);
		// 设置右边框颜色;
		style.setRightBorderColor(HSSFColor.BLACK.index);
		// 设置顶边框;
		style.setBorderTop(HSSFCellStyle.BORDER_THIN);
		// 设置顶边框颜色;
		style.setTopBorderColor(HSSFColor.BLACK.index);
		// 在样式用应用设置的字体;
		style.setFont(font);
		// 设置自动换行;
		style.setWrapText(false);
		// 设置水平对齐的样式为居中对齐;
		style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
		// 设置垂直对齐的样式为居中对齐;
		style.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);

		return style;

	}

	/**
	 * 读取Excel数据
	 * @param inputStream
	 * @param startRowNum
	 * @return
	 */
	public static List<List<String>> readExcel(InputStream inputStream, int startRowNum) {
		return readExcel(inputStream, startRowNum, 0);
	}
	
	/**
	 * 读取Excel数据   2007版本以上
	 * @param inputStream
	 * @param startRowNum
	 * 
	 * @return
	 */
	public static List<List<String>> readExcel(InputStream inputStream, int startRowNum, int sheetIdx,String highEdition) {
		List<List<String>> lists = null;
		XSSFWorkbook hssfWorkbook = null;
		try {
			hssfWorkbook = new XSSFWorkbook(inputStream);

			XSSFSheet hssfSheet = hssfWorkbook.getSheetAt(sheetIdx);
			if (null != hssfSheet) {
				lists = Lists.newArrayList();
				for (int rowNum = startRowNum; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
					XSSFRow hssfRow = hssfSheet.getRow(rowNum);
					if (hssfRow != null) {
						List<String> list = Lists.newArrayList();
						for (int cellNum = 0; cellNum < hssfRow.getLastCellNum(); cellNum++) {
							XSSFCell hssfCell = hssfRow.getCell(cellNum);
							if (null != hssfCell) {
							    //日期格式取值
                                if(Cell.CELL_TYPE_NUMERIC==hssfCell.getCellType() && DateUtil.isCellDateFormatted(hssfCell)){
                                    SimpleDateFormat dff = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                    String date = dff.format(hssfCell.getDateCellValue());
                                    list.add(date);
							        continue;
                                }
								hssfCell.setCellType(Cell.CELL_TYPE_STRING);
								list.add(hssfCell.getStringCellValue());
							} else {
								list.add("-");
							}
						}
						lists.add(list);
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return lists;
	}
	
	
	/**
	 * 读取Excel数据
	 * @param inputStream
	 * @param startRowNum
	 * @return
	 */
	public static List<List<String>> readExcel(InputStream inputStream, int startRowNum, int sheetIdx) {
		List<List<String>> lists = null;
		HSSFWorkbook hssfWorkbook = null;
		try {
			hssfWorkbook = new HSSFWorkbook(inputStream);

			HSSFSheet hssfSheet = hssfWorkbook.getSheetAt(sheetIdx);
			if (null != hssfSheet) {
				lists = Lists.newArrayList();
				for (int rowNum = startRowNum; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
					HSSFRow hssfRow = hssfSheet.getRow(rowNum);
					if (hssfRow != null) {
						List<String> list = Lists.newArrayList();
						for (int cellNum = 0; cellNum < hssfRow.getLastCellNum(); cellNum++) {
							HSSFCell hssfCell = hssfRow.getCell(cellNum);
							if (null != hssfCell) {
							    //日期格式取值
                                if(Cell.CELL_TYPE_NUMERIC==hssfCell.getCellType() && DateUtil.isCellDateFormatted(hssfCell)){
                                    SimpleDateFormat dff = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                    String date = dff.format(hssfCell.getDateCellValue());
                                    list.add(date);
							        continue;
                                }
								hssfCell.setCellType(Cell.CELL_TYPE_STRING);
								list.add(hssfCell.getStringCellValue());
							} else {
								list.add("-");
							}
						}
						lists.add(list);
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return lists;
	}


	/**
	 * 读取Excel数据
	 * 
	 * @param inputStream
	 * @param startRowNum
	 * @return
	 */
	public static List<List<List<String>>> readExcel(InputStream inputStream, int startRowNum,
			Integer[] sheetIdxs) {
		List<List<List<String>>> lists = new ArrayList<>();
		HSSFWorkbook hssfWorkbook = null;
		try {
			hssfWorkbook = new HSSFWorkbook(inputStream);
			for (Integer i = 0; i < sheetIdxs.length; i++) {
				HSSFSheet hssfSheet = hssfWorkbook.getSheetAt(sheetIdxs[i]);
				List<List<String>> excelList = null;
				if (null != hssfSheet) {
					excelList = Lists.newArrayList();
					for (int rowNum = startRowNum; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
						HSSFRow hssfRow = hssfSheet.getRow(rowNum);
						if (hssfRow != null) {
							List<String> list = Lists.newArrayList();
							for (int cellNum = 0; cellNum < hssfRow.getLastCellNum(); cellNum++) {
								HSSFCell hssfCell = hssfRow.getCell(cellNum);
								if (null != hssfCell) {
									// 日期格式取值
									if (Cell.CELL_TYPE_NUMERIC == hssfCell.getCellType()
											&& DateUtil.isCellDateFormatted(hssfCell)) {
										SimpleDateFormat dff = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
										String date = dff.format(hssfCell.getDateCellValue());
										list.add(date);
										continue;
									}
									hssfCell.setCellType(Cell.CELL_TYPE_STRING);
									list.add(hssfCell.getStringCellValue());
								} else {
									list.add("-");
								}
							}
							excelList.add(list);
						}
					}
					lists.add(excelList);
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return lists;
	}
	
	
	/**
	 * 读取EXCEL数据，根据表头名称进行封装数据集
	 * @param inputStream
	 * @return
	 * @throws Exception
	 */
	public static List<Map<String,Object>> readExcel(InputStream inputStream) throws Exception{
		
		List<Map<String,Object>> excelList = new ArrayList<Map<String, Object>>();

        HSSFWorkbook hssfWorkbook = null;

        hssfWorkbook = new HSSFWorkbook(inputStream);       
        //获取表头信息
        Map<String,Integer> heardMap=getExcelTableHeard(hssfWorkbook);	
		//封装结果集
        excelList=getExcelTable(hssfWorkbook,heardMap);
       
		return excelList;
	}
	
	
	/**
	 * 获取表头字段
	 * @param hssfWorkbook
	 * @return
	 */
	private static Map<String,Integer> getExcelTableHeard(HSSFWorkbook hssfWorkbook){
		
	    Map<String,Integer> heardMap = new HashMap<String,Integer>();		
		HSSFSheet hssfSheet = hssfWorkbook.getSheetAt(0);
        int i =hssfSheet.getFirstRowNum();
        HSSFRow hssfRow = hssfSheet.getRow(i);       
        for (int cellNum = 0; cellNum < hssfRow.getLastCellNum(); cellNum++) {
            HSSFCell hssfCell = hssfRow.getCell(cellNum);
            hssfCell.setCellType(Cell.CELL_TYPE_STRING);
            String value=hssfCell.getStringCellValue();
            if(!value.isEmpty()){
                heardMap.put(value,cellNum);
            }
        }		
		return heardMap;
	}
	
	/**
	 * 根据表头字段封装
	 * @param hssfWorkbook
	 * @param heardMap
	 * @return
	 */
	private static List<Map<String,Object>> getExcelTable(HSSFWorkbook hssfWorkbook,Map<String,Integer> heardMap){
		
		List<Map<String,Object>> excelList = new ArrayList<Map<String, Object>>();
		
		HSSFSheet hssfSheet = hssfWorkbook.getSheetAt(0);
        int i =hssfSheet.getFirstRowNum();
        int startRowNum=i+1;
		for (int rowNum = startRowNum; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
			HSSFRow hssfRow = hssfSheet.getRow(rowNum);
			if (hssfRow != null) {
				 Map<String,Object> cellMap = new HashMap<String, Object>();
				for (int cellNum = 0; cellNum < hssfRow.getLastCellNum(); cellNum++) {
					HSSFCell hssfCell = hssfRow.getCell(cellNum);
					if (null != hssfCell) {						
						if (Cell.CELL_TYPE_NUMERIC == hssfCell.getCellType()
								&& DateUtil.isCellDateFormatted(hssfCell)) {
							SimpleDateFormat dff = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
							String date = dff.format(hssfCell.getDateCellValue());									
							for (Map.Entry<String,Integer> entry:heardMap.entrySet()) {
		                        int cell=entry.getValue();
		                        if(cell==cellNum){
		                            cellMap.put(entry.getKey(),date);
		                        }
		                    }
							continue;
						}
						hssfCell.setCellType(Cell.CELL_TYPE_STRING);						
						for (Map.Entry<String,Integer> entry:heardMap.entrySet()) {
	                        int cell=entry.getValue();
	                        if(cell==cellNum){
	                            cellMap.put(entry.getKey(),hssfCell.getStringCellValue());
	                        }
	                    }					
					}else {
						for (Map.Entry<String,Integer> entry:heardMap.entrySet()) {
	                        int cell=entry.getValue();
	                        if(cell==cellNum){
	                            cellMap.put(entry.getKey(),null);
	                        }
	                    }	
					}
					
				}
				excelList.add(cellMap);
			}
		}
		return excelList;
	}
	
	
	
}
